from StringIO import StringIO

from util import *

log = logging.getLogger("cumin.admin")

schema_version = "1.0"

class CuminAdmin(object):
    def __init__(self, app):
        self.app = app

    def get_schema(self):
        writer = StringIO()
        self.app.model.sql_model.write_create_ddl(writer)
        return writer.getvalue()

    def create_schema(self, cursor):
        cursor.execute(self.get_schema())

        cls = self.app.model.com_redhat_cumin.Info

        obj = cls.create_object(cursor)
        obj.schema_version = schema_version
        obj.fake_qmf_values()
        obj.save(cursor)

    def check_schema(self, cursor):
        cls = self.app.model.com_redhat_cumin.Info
        info = cls.get_object(cursor)

        if not info:
            raise Exception("The schema isn't there")

        if info.schema_version != schema_version:
            args = (schema_version, info.schema_version)
            msg = "Expected schema version %s; found version %s" % args
            raise Exception(msg)

        return info.schema_version

    def drop_schema(self, cursor):
        writer = StringIO()
        self.app.model.sql_model.write_drop_ddl(writer)
        sql = writer.getvalue()

        cursor.execute(sql)

    def get_role(self, cursor, name):
        cls = self.app.model.com_redhat_cumin.Role
        return cls.get_object(cursor, name=name)

    def add_role(self, cursor, name):
        cls = self.app.model.com_redhat_cumin.Role

        role = cls.create_object(cursor)
        role.name = name
        role.fake_qmf_values()
        role.save(cursor)

        return role

    def get_user(self, cursor, name):
        cls = self.app.model.com_redhat_cumin.User
        return cls.get_object(cursor, name=name)

    def add_user(self, cursor, name, crypted_password):
        cls = self.app.model.com_redhat_cumin.User

        user = cls.create_object(cursor)
        user.name = name
        user.password = crypted_password
        user.fake_qmf_values()
        user.save(cursor)

        return user

    def get_assignment(self, cursor, user, role):
        cls = self.app.model.com_redhat_cumin.UserRoleMapping

        mapping = cls.get_object(cursor, _user_id=user._id, _role_id=role._id)

        return mapping

    def add_assignment(self, cursor, user, role):
        cls = self.app.model.com_redhat_cumin.UserRoleMapping

        mapping = cls.create_object(cursor)
        mapping._user_id = user._id
        mapping._role_id = role._id
        mapping.fake_qmf_values()
        mapping.save(cursor)

        return mapping
