import psycopg2
import re
import threading

from psycopg2.extensions import *

from util import *

log = logging.getLogger("cumin.database")

class CuminDatabase(object):
    def __init__(self, app, dsn):
        self.app = app
        self.dsn = dsn

        self.connection_args = dict()
        self.thread_local = threading.local()

    def init(self):
        log.info("Initializing %s", self)

        #m = re.match(r"^([^:]+)://([^@]+)@([^/]+)/(.+)$", self.uri)

    def get_connection(self):
        return psycopg2.connect(self.dsn)

    def get_read_connection(self):
        key = "read_connection"
        conn = getattr(self.thread_local, key, None)

        if not conn or conn.closed:
            conn = self.get_connection()
            setattr(self.thread_local, key, conn)

            conn.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)

        return conn

    def get_read_cursor(self):
        return self.get_read_connection().cursor()

    def __repr__(self):
        return self.__class__.__name__

def modify_pghba_conf(path, database_name, user_name):
    comment_or_empty_line_pattern = re.compile('^\w*#|^\w*$')
    record_pattern = re.compile('^\w*(local|host|hostssl|hostnossl)')

    file = open(path, "r")

    lines = list()
    first_record_index = None

    for i, line in enumerate(file):
        lines.append(line)

        if record_pattern.match(line):
            if first_record_index is None:
                first_record_index = i

            tokens = line.split()

            if tokens[1] == database_name:
                raise Exception("This file already contains a " + \
                                    "%s record" % database_name)
        elif comment_or_empty_line_pattern.match(line):
            pass
        else:
            raise Exception("This doesn't look like a pg_hba.conf file")

    file.close()

    if first_record_index is None:
        first_record_index = len(lines)

    line = "host %s %s ::1/128 trust\n" % (database_name, user_name)
    lines.insert(first_record_index, line)
    line = "host %s %s 127.0.0.1/32 trust\n" % (database_name, user_name)
    lines.insert(first_record_index, line)

    file = open(path, "w")

    for line in lines:
        file.write(line)

    file.close()
