from update import *
from util import *

log = logging.getLogger("mint.expire")

class ExpireThread(MintPeriodicProcessThread):
    def __init__(self, app):
        super(ExpireThread, self).__init__(app, 60 * 60)

        self.threshold = 24 * 3600

    def init(self):
        super(ExpireThread, self).init()

        log.debug("Expire interval is %i seconds", self.interval)
        log.debug("Expire threshold is %i seconds", self.threshold)

    def run(self):
        if self.interval >= 60 * 60:
            time.sleep(0.25 * self.interval)
        else:
            time.sleep(self.interval)
        super(ExpireThread, self).run()

    def process(self):
        threshold = self.threshold
        count = 0

        log.info("Starting expire")

        conn = self.app.database.get_connection()

        try:
            cursor = conn.cursor()

            for pkg in self.app.model._packages:
                for cls in pkg._classes:
                    if cls._storage == "none":
                        continue

                    values = {"seconds": threshold}
                    cls.sql_samples_delete.execute(cursor, values)

                    conn.commit()

                    log.debug("Deleted %i %s", cursor.rowcount, cls)

                    count += cursor.rowcount
        finally:
            conn.close()

        log.info("Expired %i samples", count)
