import htmlentitydefs as entity
import logging
import math
import os
import random
import sys
import time

from copy import copy
from cStringIO import StringIO
from datetime import datetime, timedelta
from threading import Thread
from traceback import print_exc
from urllib import quote, unquote_plus, unquote
from urlparse import urlsplit
from xml.sax.saxutils import escape

from parsley.collectionsex import *

def unique_id():
    bits0 = random.getrandbits(32)
    bits1 = random.getrandbits(32)
    bits2 = random.getrandbits(32)
    bits3 = random.getrandbits(32)

    return "%08x-%08x-%08x-%08x" % (bits0, bits1, bits2, bits3)

def escape_amp(string):
    return str(string).replace("&", "&amp;")

def escape_entity(string, exceptions=None):
    if not string:
        return ""
    t = ""
    ex = exceptions or list()
    for i in string:
        if ord(i) in entity.codepoint2name and \
            i not in ex:
            name = entity.codepoint2name.get(ord(i))
            t += "&" + name + ";"
        else:
            t += i
    return t

class Writer(object):
    def __init__(self):
        self.writer = StringIO()

    def write(self, string):
        self.writer.write(string)

    def to_string(self):
        string = self.writer.getvalue()

        self.writer.close()

        return string

class StringCatalog(object):
    def __init__(self, file):
        self.strings = None
        self.path = os.path.splitext(file)[0] + ".strings"

    def clear(self):
        self.strings = None

    def load(self):
        file = open(self.path)
        try:
            self.strings = self.parse_catalog_file(file)
        finally:
            file.close()

    def parse_catalog_file(self, file):
        strings = dict()
        key = None
        writer = Writer()

        for line in file:
            line = line.rstrip()

            if line.startswith("[") and line.endswith("]"):
                if key:
                    strings[key] = writer.to_string().strip()

                writer = Writer()

                key = line[1:-1]

                continue

            writer.write(line)
            writer.write("\r\n")

        strings[key] = writer.to_string().strip()

        return strings

    def get(self, key):
        if not self.strings:
            self.load()

        return self.strings.get(key)

    def __repr__(self):
        return "%s('%s')" % (self.__class__.__name__, self.path)

